package de.bsvrz.vew.bmvew.bmvew.regel;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dambach.vewdynobj.VerwaltungDynObj;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.bmvew.bmvew.BetriebsMeldungApp;
import de.bsvrz.vew.bmvew.bmvew.meldung.InformationsAnteil;
import de.bsvrz.vew.bmvew.bmvew.meldung.Meldung;

/**
 * Die Klasse implementiert das Ausfuehr-Verhalten Verwalten
 * 
 * @version $Revision: 1.6 $ / $Date: 2015/06/09 13:02:12 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 * 
 */
public class AusfuehrenVerwalten implements AktionAusfuehrVerhalten
{

  /**
   * Der Debugger
   */
  Debug _debug;

  /**
   * Das Feld mit Parametertexten
   */
  TextArray _parameter;

  /**
   * Der Aktionstetxt
   */
  TextValue _aktion;

  /**
   * Konstruktor
   * 
   * @param parameter
   *          Feld mit Parametertexten
   * @param aktion
   *          Aktionstetxt
   * 
   */
  public AusfuehrenVerwalten(TextArray parameter, TextValue aktion)
  {
    // TODO Auto-generated constructor stub
    if (aktion == null || parameter == null)
      throw new IllegalArgumentException();

    _parameter = parameter;
    _aktion = aktion;
    _debug = Debug.getLogger();
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.bsvrz.vew.bmvew.bmvew.regel.AktionAusfuehrVerhalten#ausfuehrenAktion(de.bsvrz.dav.daf.main.ResultData)
   */
  public boolean ausfuehrenAktion(final ResultData betriebsMeldung)
  {
    String kennung = betriebsMeldung.getData().getItem("ApplikationsKennung").valueToString();
    String id = betriebsMeldung.getData().getItem("ID").valueToString();
    String pid = "meldung." + kennung + id;
    String name = "Meldung" + kennung + id;

    if (pid.length() >= 254)
      pid = pid.substring(0, 254);

    if (name.length() >= 254)
      name = name.substring(0, 254);

    Integer wichtigkeit = null;
    String[] meldungsGruppen = new String[_parameter.getLength() - 1];

    for (int i = 0; i < _parameter.getLength(); i++)
    {
      String s = _parameter.getText(i);
      if (i == 0)
      {
        wichtigkeit = new Integer(s);
      }
      else if (i > 0)
      {
        meldungsGruppen[i - 1] = s;
      }
    }

    Integer wichtig = wichtigkeit;

    ClientDavInterface con = BetriebsMeldungApp.getConnection();
    DataModel modell = con.getDataModel();

    ConfigurationArea bereich = BetriebsMeldungApp.getKonfigurationsBereich();
    ConfigurationObject conf = BetriebsMeldungApp.getKonfigurationsObjekt();

    DynamicObjectType dot = (DynamicObjectType)modell.getType("typ.meldung");
    AttributeGroup atg = modell.getAttributeGroup("atg.meldungEigenschaften");
    Aspect asp = modell.getAspect("asp.eigenschaften");

    VerwaltungDynObj vew = new VerwaltungDynObj(con, modell, bereich, dot, conf, atg, asp);

    Data data = con.createData(atg);
    data.getTextValue("ApplikationsKennung").setText(kennung);
    data.getTextValue("ID").setText(id);
    Data[] datas = new Data[1];
    datas[0] = data;

    // if (!pruefeMeldungVorhanden(kennung, id))
    if (!pruefeMeldungVorhanden(pid))
    {
      // erzeuge neues Meldungsobjekt

      _debug.fine("Aktion -> " + _aktion.getText() + " erzeuge Objekt: " + pid);

      // if (pid.length() >= 255)
      // pid = pid.substring(0, 10);
      //
      // if (name.length() >= 255)
      // name = name.substring(0, 10);

      vew.erzeuge(pid, name, "Meldungen", datas);

      atg = modell.getAttributeGroup("atg.meldung");
      asp = modell.getAspect("asp.parameterVorgabe");
      data = con.createData(atg);

      data.getTextValue("Wichtigkeit").setText(wichtig.toString());
      data.getTextValue("Status").setText("neu");

      Calendar cal = new GregorianCalendar().getInstance();

      // Der erste Tag des aktuellen Jahres wird eingetragen
      data.getTimeValue("Wiedervorlage").setText("01.01." + cal.get(Calendar.YEAR));
      data.getTextValue("Ergnzungstext").setText("");

      // Long zeit = betriebsMeldung.getDataTime();
      Long zeit = System.currentTimeMillis();
      Data dataBetriebsMeldung = betriebsMeldung.getData();
      TextValue typ = dataBetriebsMeldung.getItem("MeldungsTyp").asTextValue();
      TextValue klasse = dataBetriebsMeldung.getItem("MeldungsKlasse").asTextValue();
      Array refs = dataBetriebsMeldung.getItem("Referenz").asArray();
      ReferenceValue ref = null;
      if (refs.getLength() > 0)
        ref = refs.getItem(0).asReferenceValue();
      TextValue status = dataBetriebsMeldung.getItem("Status").asTextValue();
      Data urlasser = dataBetriebsMeldung.getItem("Urlasser");
      TextValue text = dataBetriebsMeldung.getItem("MeldungsText").asTextValue();
      Array a = data.getItem("InformationsMeldungsAnteil").asArray();
      a.setLength(1);
      Data infoAnteil = a.getItem(0);
      infoAnteil.getTimeValue("Zeitstempel").setMillis(zeit);
      infoAnteil.getTextValue("MeldungsTyp").setText(typ.getText());
      infoAnteil.getTextValue("MeldungsKlasse").setText(klasse.getText());
      if (refs.getLength() > 0)
      {

        Array refsNeu = infoAnteil.getItem("Referenz").asArray();
        refsNeu.setLength(1);
        refsNeu.getItem(0).asReferenceValue().setSystemObject(refs.getItem(0).asReferenceValue().getSystemObject());

      }
      infoAnteil.getTextValue("MeldungsStatus").setText(status.getText());
      infoAnteil.getTextValue("MeldungsText").setText(text.getText());
      Data urlasserNeu = infoAnteil.getItem("Urlasser");
      urlasserNeu.getItem("BenutzerReferenz").asReferenceValue()
          .setSystemObject(urlasser.getItem("BenutzerReferenz").asReferenceValue().getSystemObject());
      urlasserNeu.getItem("Ursache").asTextValue().setText(urlasser.getItem("Ursache").asTextValue().getText());
      urlasserNeu.getItem("Veranlasser").asTextValue().setText(urlasser.getItem("Veranlasser").asTextValue().getText());

      InformationsAnteil anteil = new InformationsAnteil(zeit, typ, klasse, ref, status, text, urlasser);

      // publiziere
      _debug.fine("Aktion -> " + _aktion.getText() + " publiziere " + data + " fr Meldung: "
          + modell.getObject(pid).getPid());

      atg = modell.getAttributeGroup("atg.meldung");
      asp = modell.getAspect("asp.zustand");
      DataDescription dd = new DataDescription(atg, asp);

      Vector<ResultData> sendeDaten = BetriebsMeldungApp.getSendeDaten();
      ResultData res = new ResultData(modell.getObject(pid), dd, System.currentTimeMillis(), data);

      synchronized (sendeDaten)
      {
        sendeDaten.add(res);
        sendeDaten.notify();
      }

      // Sender sender = new Sender(con, SenderRole.source());
      // sender.sende(modell.getObject(pid), atg, asp, data);

      atg = modell.getAttributeGroup("atg.meldung");
      asp = modell.getAspect("asp.parameterVorgabe");
      dd = new DataDescription(atg, asp);
      datas[0] = data;
      vew.setDynamicObject((DynamicObject)modell.getObject(pid));
      _debug.fine("Aktion -> " + _aktion.getText() + " parametriere Objekt: " + pid);

      ResultData result = new ResultData(modell.getObject(pid), dd, System.currentTimeMillis(), data);

      Vector<ResultData> paramDaten = BetriebsMeldungApp.getParamDaten();

      synchronized (paramDaten)
      {
        paramDaten.add(result);
        paramDaten.notify();
      }

      Map<String, Meldung> meldungList = BetriebsMeldungApp.getMeldungList();

      // synchronized (meldungList)
      // {
      // try
      // {
      // _debug.fine("Warte auf Parameter: " + pid);
      //
      // if (!meldungList.containsKey(pid))
      // {
      // meldungList.wait();
      // }
      // }
      // catch (InterruptedException e)
      // {
      // // TODO Auto-generated catch block
      // e.printStackTrace();
      // }
      // }

      synchronized (meldungList)
      {
        try
        {
          if (!meldungList.containsKey(pid))
          {
            meldungList.wait(10000); // maximal 10 Sekunden auf die Parameter warten, sonst nichts mehr machen
          }
        }
        catch (InterruptedException e1)
        {
          e1.printStackTrace();
        }
      }

      if (!meldungList.containsKey(pid))
      {
        _debug.error("Konnte Parameter nicht lesen: " + pid);
        return false;
      }

      _debug.fine("Parameter erhalten");

      Meldung meldung = BetriebsMeldungApp.getMeldungList().get(pid);

      // Dies passiert schon in der beim Erzeugen des Meldungsobjektes beim Eintragen in
      // die globale Meldungsliste

      // Infoanteil zur Liste der Meldung inkrementieren
      // synchronized (mutex)
      // {
      // meldung.getInfoAnteilListe().push(anteil);
      //
      // }

      if (meldung != null)
      {
        TextValue tv = data.getItem("Wichtigkeit").asTextValue();
        tv.setText(wichtig.toString());

        meldung.setWichtigkeit(tv);

        tv = data.getItem("Status").asTextValue();
        if (tv.getText().equals("keine nderung"))
          tv.setText(meldung.getEndlicherAutomat().getMomentZustand().toString());
        meldung.setStatus(tv);
        meldung.setWiedervorlage(data.getTimeValue("Wiedervorlage").getMillis());

        meldung.setData(data);
      }
      else
      {
        _debug.error("Konnte Meldungsdaten nicht schreiben: " + pid);
        return false;
      }

      for (String gruppe : meldungsGruppen)
      {

        HashSet<String> set = new HashSet<String>();
        set.add(pid);

        Map<String, Set<String>> map = BetriebsMeldungApp.getMeldungsGruppenZuordnung();
        if (map.containsKey(gruppe))
        {
          Set<String> set2 = map.get(gruppe);
          for (String s : set2)
          {
            set.add(s);
          }

        }

        atg = BetriebsMeldungApp.getConnection().getDataModel().getAttributeGroup("atg.meldungsGruppeMeldung");
        asp = BetriebsMeldungApp.getConnection().getDataModel().getAspect("asp.zustand");
        dd = new DataDescription(atg, asp);
        Data d = BetriebsMeldungApp.getConnection().createData(atg);

        Array a1 = d.getItem("MeldungReferenz").asArray();

        a1.setLength(set.size());

        int i = 0;
        for (String s : set)
        {
          SystemObject so = BetriebsMeldungApp.getConnection().getDataModel().getObject(s);
          a1.getItem(i).asReferenceValue().setSystemObject(so);
          i++;
        }

        SystemObject so = BetriebsMeldungApp.getConnection().getDataModel().getObject(gruppe);

        sendeDaten = BetriebsMeldungApp.getSendeDaten();
        ResultData res1 = new ResultData(so, dd, System.currentTimeMillis(), d);

        synchronized (sendeDaten)
        {
          _debug.fine("Aktion -> Meldungsgruppen" + " versendeZustand: " + d);
          sendeDaten.add(res1);
          sendeDaten.notify();
        }

        // Sender sender = BetriebsMeldungApp.getConnection(), SenderRole.source());
        // sender.sende(so, atg, asp, d);

        asp = BetriebsMeldungApp.getConnection().getDataModel().getAspect("asp.parameterVorgabe");

        ResultData res2 = new ResultData(so, new DataDescription(atg, asp), System.currentTimeMillis(), d);

        synchronized (BetriebsMeldungApp.getParamDaten())
        {
          _debug.fine("Aktion -> Meldungsgruppen" + " parametriere : " + d);
          BetriebsMeldungApp.getParamDaten().add(res2);
          BetriebsMeldungApp.getParamDaten().notify();
        }

        // Map<String, Set<String>> map = BetriebsMeldungApp.getMeldungsGruppenZuordnung();
        map.put(gruppe, set);
        meldung.getMeldungsGruppen().add(gruppe);

      }

    }
    else
    // update
    {
      // SystemObject so = con.getDataModel().getObject("meldung." + kennung + id);
      SystemObject so = con.getDataModel().getObject(pid);

      if (so != null)
      {

        if (BetriebsMeldungApp.getMeldungList().containsKey(so.getPid()))
        {
          // Daten aus der Betriebsmeldung holen
          // Long zeit = betriebsMeldung.getDataTime();
          Long zeit = System.currentTimeMillis();
          data = betriebsMeldung.getData();

          TextValue typ = data.getItem("MeldungsTyp").asTextValue();
          TextValue klasse = data.getItem("MeldungsKlasse").asTextValue();

          Array refs = data.getItem("Referenz").asArray();

          ReferenceValue ref = null;
          if (refs.getLength() > 0)
            ref = refs.getItem(0).asReferenceValue();

          TextValue status = data.getItem("Status").asTextValue();

          Data urlasser = data.getItem("Urlasser");
          TextValue text = data.getItem("MeldungsText").asTextValue();

          // Informationsanteil erzeugen
          InformationsAnteil anteil = new InformationsAnteil(zeit, typ, klasse, ref, status, text, urlasser);
          // Meldung holen
          Meldung meldung = BetriebsMeldungApp.getMeldungList().get(so.getPid());

          // Infoanteil zur Liste der Meldung inkrementieren
          meldung.getInfoAnteilListe().push(anteil);

          atg = modell.getAttributeGroup("atg.meldung");
          asp = modell.getAspect("asp.zustand");
          DataDescription dd = new DataDescription(atg, asp);

          // Arbeitskopie des Datums der atg erstellen
          data = meldung.getData().createModifiableCopy();

          Array array = data.getItem("InformationsMeldungsAnteil").asArray();

          InformationsAnteil[] all;
          all = meldung.getInfoAnteilListe().getAll();

          array.setLength(all.length);

          for (int i = 0; i < all.length; i++)
          {
            array.getItem(i).getTimeValue("Zeitstempel").setMillis(all[i].getZeit());
            array.getItem(i).getTextValue("MeldungsTyp").setText(all[i].getTyp().getText());
            array.getItem(i).getTextValue("MeldungsKlasse").setText(all[i].getKlasse().getText());

            if (all[i].getReferenz() != null)
            {

              refs = array.getItem(i).getItem("Referenz").asArray();
              refs.setLength(1);
              refs.getItem(0).asReferenceValue().setSystemObject(all[i].getReferenz().getSystemObject());

            }

            array.getItem(i).getTextValue("MeldungsStatus").setText(all[i].getStatus().getText());
            array.getItem(i).getTextValue("MeldungsText").setText(all[i].getText().getText());

            array.getItem(i).getItem("Urlasser").getItem("BenutzerReferenz").asReferenceValue()
                .setText(all[i].getUrlasser().getItem("BenutzerReferenz").asReferenceValue().getText());
            array.getItem(i).getItem("Urlasser").getItem("Ursache").asTextValue()
                .setText(all[i].getUrlasser().getItem("Ursache").asTextValue().getText());
            array.getItem(i).getItem("Urlasser").getItem("Veranlasser").asTextValue()
                .setText(all[i].getUrlasser().getItem("Veranlasser").asTextValue().getText());

          }

          data.getItem("Urlasser").getItem("BenutzerReferenz").asReferenceValue()
              .setText(meldung.getUrlasser().getItem("BenutzerReferenz").asReferenceValue().getText());
          data.getItem("Urlasser").getItem("Ursache").asTextValue()
              .setText(meldung.getUrlasser().getItem("Ursache").asTextValue().getText());
          data.getItem("Urlasser").getItem("Veranlasser").asTextValue()
              .setText(meldung.getUrlasser().getItem("Veranlasser").asTextValue().getText());

          if (status.getText().equals("Gutmeldung"))
          {

            // TextValue statusNeu = data.getItem("Status").asTextValue();
            //
            // statusNeu.setText("erledigt");
            //
            // if (!meldung.setStatus(statusNeu))
            // {
            // _debug.fine("Zustand konnte nicht verndert werden");
            // return false;
            // }
            // else
            if (meldung != null && meldung.getStatus() != null)
              _debug.fine("Gutmeldung erhalten -> Status der Meldung: " + meldung.getStatus().getText());

          }

          // Umkopieren der Betriebsmeldungsdaten in die Arbeitskopie ist beendet,
          // es kann nun der Datensatz der Meldung aktualisiert werden
          meldung.setData(data);

          // else
          {

            // // publiziere
            _debug.fine("Aktion -> " + _aktion.getText() + " publiziere " + data + " fr Meldung: " + so.getPid());

            Vector<ResultData> sendeDaten = BetriebsMeldungApp.getSendeDaten();
            ResultData res = new ResultData(so, dd, System.currentTimeMillis(), data);

            synchronized (sendeDaten)
            {
              sendeDaten.add(res);
              sendeDaten.notify();
            }

            // Sender sender = new Sender(con, SenderRole.source());
            // sender.sende(so, atg, asp, data);

            atg = modell.getAttributeGroup("atg.meldung");
            asp = modell.getAspect("asp.parameterVorgabe");
            dd = new DataDescription(atg, asp);

            _debug.fine("Aktion -> " + _aktion.getText() + " persistiere Daten fr Meldung: " + so.getPid());

            ResultData result = new ResultData(so, dd, System.currentTimeMillis(), data);

            Vector<ResultData> paramDaten = BetriebsMeldungApp.getParamDaten();

            synchronized (paramDaten)
            {
              paramDaten.add(result);
              paramDaten.notify();
            }

          }

        }
        else
        {
          _debug.error("Objekt: " + so.getPid() + " ist nicht in MeldungListe vorhanden!");

        }

      }
      else
      {

        _debug.error("Objekt: " + kennung + " : " + id + " ist null!");

      }

    }

    return false;
  }

  /**
   * Prft ob die Meldung schon vorhanden ist
   * 
   * @param kennung
   *          Kobmination aus Applikationskennung und ID
   * @param id
   * 
   * @return true, wenn Meldung schon vorhanden ist
   */
  private boolean pruefeMeldungVorhanden(String kennung, String id)
  {

    ClientDavInterface con = BetriebsMeldungApp.getConnection();
    SystemObject so = con.getDataModel().getObject("meldung." + kennung + id);
    if (so != null)
    {
      if (so.isValid())
      {

        if (so.getPid().length() > 0)
        {

          _debug.fine(so.getPid() + " Objekt schon vorhanden");

          return true;

        }
        else
        {

          _debug.fine("Lnge Pid ist null");

          return true;

        }

      }
      else
      {

        _debug.fine(so.getPid() + " Objekt ist schon vorhanden, aber ungueltig");

        return false;
      }

    }
    else
    {
      _debug.fine("meldung." + kennung + id + " Objekt nicht vorhanden");
    }
    return false;
  }

  /**
   * Prft ob die Meldung schon vorhanden ist
   * 
   * @param kennung
   *          Kobmination aus Applikationskennung und ID
   * @param id
   * 
   * @return true, wenn Meldung schon vorhanden ist
   */
  private boolean pruefeMeldungVorhanden(String pid)
  {

    ClientDavInterface con = BetriebsMeldungApp.getConnection();
    SystemObject so = con.getDataModel().getObject(pid);
    if (so != null)
    {
      if (so.isValid())
      {

        if (so.getPid().length() > 0)
        {

          _debug.fine(so.getPid() + " Objekt schon vorhanden");

          return true;

        }
        else
        {

          _debug.fine("Lnge Pid ist null");

          return true;

        }

      }
      else
      {

        _debug.fine(so.getPid() + " Objekt ist schon vorhanden, aber ungueltig");

        return false;
      }

    }
    else
    {
      _debug.fine("meldung." + pid + " Objekt nicht vorhanden");
    }
    return false;
  }

}
